from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import shutil

import tensorflow as tf
from tensorflow.keras.losses import SparseCategoricalCrossentropy

from data_utils import read_data
from data_utils import create_batch_tf_dataset
from models import create_model
from utils import DEFINE_boolean
from utils import DEFINE_float
from utils import DEFINE_integer
from utils import DEFINE_string
from utils import print_user_flags

flags = tf.compat.v1.flags
FLAGS = flags.FLAGS

DEFINE_boolean("reset_output_dir", False, "Delete output_dir if exists.")
DEFINE_string("data_path", "../data/cifar10", "Path to CIFAR-10 data")
DEFINE_string("output_dir", "output", "Path to log folder")
DEFINE_string("model_name", "conv",
              "Name of the method. [naive|mlp|conv]")
DEFINE_integer("n_epochs", 20, "How many epochs to train in total")
# DEFINE_integer("train_steps", 10000, "How many steps to train in total")
DEFINE_integer("validate_freq", 2, "Frequency of validation")
DEFINE_integer("n_classes", 10, "Number of classes")
DEFINE_integer("batch_size", 32, "Batch size")
DEFINE_float("lr", 1e-3, "Init learning rate")
DEFINE_float("reg_coeff", 1e-2, "Regularization coefficient")


def main():
  print("-" * 80)
  if not os.path.isdir(FLAGS.output_dir):
    print("Path {0} does not exist. Creating.".format(FLAGS.output_dir))
    os.makedirs(FLAGS.output_dir)
  elif FLAGS.reset_output_dir:
    print("Path {0} exists. Remove and remake.".format(FLAGS.output_dir))
    shutil.rmtree(FLAGS.output_dir)
    os.makedirs(FLAGS.output_dir)

  print_user_flags()

  # data
  images, labels = read_data(FLAGS.data_path)
  dataset_dict = create_batch_tf_dataset(images, labels,
                                         batch_size=FLAGS.batch_size)
  # split
  train_dataset = dataset_dict["train"]
  val_dataset = dataset_dict["valid"]
  test_dataset = dataset_dict["test"]

  # create model
  model = create_model(model_type=FLAGS.model_name,
                       n_outputs=FLAGS.n_classes)

  # TODO: something below
  model.compile()

  # TRAIN HERE
  model.fit(train_dataset,
            epochs=FLAGS.n_epochs,
            validation_data=val_dataset,
            validation_freq=FLAGS.validate_freq)

  # TEST HERE
  # TODO: add your code to test


if __name__ == "__main__":
  main()
