from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import shutil

import tensorflow as tf
from tensorflow.keras.losses import SparseCategoricalCrossentropy

from data_utils import N_CLASSES_LIMIT
from data_utils import create_batch_tf_dataset
from data_utils import pickle_load_data_dict
from models import create_model
from utils import DEFINE_boolean
from utils import DEFINE_float
from utils import DEFINE_integer
from utils import DEFINE_string
from utils import print_user_flags

flags = tf.compat.v1.flags
FLAGS = flags.FLAGS

DEFINE_boolean("reset_output_dir", False, "Delete output_dir if exists.")
DEFINE_string("pickle_path", "data_dict.pickle", "Path to data")
DEFINE_string("output_dir", "output", "Path to log folder")
DEFINE_string("model_name", "vgg",
              "Name of the method. [naive|vgg]")
DEFINE_integer("n_epochs", 50, "How many epochs to run in total")
DEFINE_integer("validate_freq", 2, "Frequency of validation")
# DEFINE_integer("train_steps", 700, "How many batches per epoch")
# DEFINE_integer("log_every", 700, "How many steps to log")
DEFINE_integer("n_classes", N_CLASSES_LIMIT, "Number of classes")
DEFINE_integer("batch_size", 32, "Batch size")
DEFINE_float("lr", 1e-3, "Learning rate for optimizer")


def main():
  print("-" * 80)
  if not os.path.isdir(FLAGS.output_dir):
    print("Path {0} does not exist. Creating.".format(FLAGS.output_dir))
    os.makedirs(FLAGS.output_dir)
  elif FLAGS.reset_output_dir:
    print("Path {0} exists. Remove and remake.".format(FLAGS.output_dir))
    shutil.rmtree(FLAGS.output_dir)
    os.makedirs(FLAGS.output_dir)

  print_user_flags()

  # data
  data_dict = pickle_load_data_dict(FLAGS.pickle_path)
  dataset_dict = create_batch_tf_dataset(data_dict,
                                         batch_size=FLAGS.batch_size)
  train_dataset = dataset_dict["train"]
  val_dataset = dataset_dict["valid"]
  test_dataset = dataset_dict["test"]

  # create model
  model = create_model(model_type=FLAGS.model_name,
                       n_outputs=FLAGS.n_classes)

  model.compile(optimizer=tf.keras.optimizers.Adam(FLAGS.lr),
                loss=SparseCategoricalCrossentropy(),
                metrics=['accuracy', 'top_k_categorical_accuracy'])

  # TRAIN HERE
  model.fit(train_dataset,
            epochs=FLAGS.n_epochs,
            validation_data=val_dataset,
            validation_freq=FLAGS.validate_freq)


# TEST HERE
# TODO: add your code to test


if __name__ == "__main__":
  main()
